﻿using System;
using System.Runtime.InteropServices;

namespace Sorayuki
{
    public class AudacityPositionReader : IDisposable
    {
        [DllImport("Kernel32.dll", EntryPoint="CreateFileMappingA", CharSet=CharSet.Ansi, CallingConvention=CallingConvention.StdCall)]
        static private extern UInt32 CreateFileMappingA(UInt32 hFile, IntPtr lpAttributes, UInt32 flProtect, UInt32 dwMaximumSizeHigh, UInt32 dwMaximumSizeLow, string lpName);

        [DllImport("Kernel32.dll", EntryPoint="OpenFileMappingA", CharSet=CharSet.Ansi, CallingConvention=CallingConvention.StdCall)]
        static private extern UInt32 OpenFileMappingA(UInt32 dwAccess, UInt32 bInherit, string lpName);

        [DllImport("Kernel32.dll", EntryPoint="MapViewOfFile", CallingConvention=CallingConvention.StdCall)]
        static private extern IntPtr MapViewOfFile(UInt32 hFileMapping, UInt32 dwAccess, UInt32 dwOffsetHigh, UInt32 dwOffsetLow, UInt32 dwLen);

        [DllImport("Kernel32.dll", EntryPoint = "UnmapViewOfFile", CallingConvention = CallingConvention.StdCall)]
        static private extern UInt32 UnmapViewOfFile(IntPtr pBase);

        [DllImport("Kernel32.dll", EntryPoint = "CloseHandle", CallingConvention = CallingConvention.StdCall)]
        static private extern UInt32 CloseHandle(UInt32 handle);

        private UInt32 _hShmem;
        private IntPtr _pShmem;

        const string _shmemName = "81bdd671-bf82-43a9-8078-dd11039f8eeb";

        public AudacityPositionReader()
        {
            UInt32 hShmem = OpenFileMappingA(3, 0, _shmemName);
            if (hShmem == 0)
            {
                hShmem = CreateFileMappingA(0, IntPtr.Zero, 4, 0, 8, _shmemName);
                if (hShmem == 0)
                    throw new Exception("Can't create shared memory block");
            }

            _hShmem = hShmem;

            IntPtr pShmem = MapViewOfFile(hShmem, 2, 0, 0, 8);
            if (pShmem == IntPtr.Zero)
            {
                CloseHandle(hShmem);
                _hShmem = 0;
                throw new Exception("Can't map shared memory block");
            }
            _pShmem = pShmem;
        }

        ~AudacityPositionReader()
        {
            Dispose();
        }

        public void Dispose()
        {
            UnmapViewOfFile(_pShmem);
            CloseHandle(_hShmem);
        }

        public double Position
        {
            get
            {
                unsafe
                {
                    double* pVal = (double*)_pShmem;
                    return *pVal;
                }
            }
        }
    }
}
